#   Copyright (C) 2011-2012 by
#   Nicholas Mancuso <nick.mancuso@gmail.com>
#   All rights reserved.
#   BSD license.

__author__ = """Nicholas Mancuso (nick.mancuso@gmail.com)"""

import bioa.io.readpairs
import bioa.io.amplicon
import bioa.io.sam
from bioa.io.readpairs import *
from bioa.io.amplicon import *
from bioa.io.sam import *
